import React, { useCallback, useMemo, useState, useEffect } from 'react';
import classnames from 'classnames';
import { parseMetadata } from '@highlight-ui/utils-commons';
import { DragIcon, DragAndDropList } from '@highlight-ui/drag-and-drop';
import { Typography } from '@highlight-ui/typography';

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function () {
  __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
  };
  return __assign.apply(this, arguments);
};
function __rest(s, e) {
  var t = {};
  for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0) t[p] = s[p];
  if (s != null && typeof Object.getOwnPropertySymbols === "function") for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
    if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i])) t[p[i]] = s[p[i]];
  }
  return t;
}
function __spreadArray(to, from, pack) {
  if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
    if (ar || !(i in from)) {
      if (!ar) ar = Array.prototype.slice.call(from, 0, i);
      ar[i] = from[i];
    }
  }
  return to.concat(ar || Array.prototype.slice.call(from));
}
var styles$1 = {
  "button": "MenuItem-module__xXLWnxde__v1-2-16",
  "isSelected": "MenuItem-module__TQYZLu2-__v1-2-16",
  "isSelectedAnchor": "MenuItem-module__ARcFohB1__v1-2-16",
  "anchorButton": "MenuItem-module__7aU3Nl3W__v1-2-16",
  "titleContainer": "MenuItem-module__lFOWt-hz__v1-2-16",
  "iconContainer": "MenuItem-module__UHD7J8ND__v1-2-16",
  "title": "MenuItem-module__MQoypTGC__v1-2-16",
  "ellipsis": "MenuItem-module__En0BSLp5__v1-2-16",
  "disabled": "MenuItem-module__QFAURvQS__v1-2-16",
  "isOver": "MenuItem-module__khGuQtQV__v1-2-16",
  "canDrop": "MenuItem-module__3ew55Azr__v1-2-16"
};
var MenuVariants;
(function (MenuVariants) {
  MenuVariants["DEFAULT"] = "default";
  MenuVariants["ANCHOR"] = "anchor";
})(MenuVariants || (MenuVariants = {}));
function MenuItem(_a) {
  var _b;
  var item = _a.item,
    isSelected = _a.isSelected,
    onClick = _a.onClick,
    dragRef = _a.dragRef,
    isOver = _a.isOver,
    canDrop = _a.canDrop,
    variant = _a.variant;
  var isAnchor = variant === MenuVariants.ANCHOR;
  var selectedStyle = isAnchor ? styles$1.isSelectedAnchor : styles$1.isSelected;
  var onClickHandler = useCallback(function (e) {
    e.stopPropagation();
    onClick(item);
  }, [item, onClick]);
  return /*#__PURE__*/React.createElement("button", __assign({
    type: "button",
    disabled: item.disabled,
    onClick: onClickHandler,
    className: classnames(styles$1.button, (_b = {}, _b[styles$1.disabled] = item.disabled, _b[selectedStyle] = isSelected, _b[styles$1.isOver] = isOver && !isSelected, _b[styles$1.canDrop] = canDrop && !isSelected, _b[styles$1.anchorButton] = isAnchor, _b))
  }, parseMetadata(item.metadata)), /*#__PURE__*/React.createElement("div", {
    className: styles$1.iconContainer
  }, dragRef && /*#__PURE__*/React.createElement(DragIcon, {
    dragRef: dragRef
  }), /*#__PURE__*/React.createElement("div", {
    className: styles$1.titleContainer
  }, /*#__PURE__*/React.createElement(Typography, {
    token: isSelected ? 'body-strong-base' : 'body-base',
    component: "span",
    className: classnames(styles$1.ellipsis)
  }, item.title))));
}
var styles = {
  "droppableZoneWrapper": "CustomMenu-module__94Eu-h5o__v1-2-16",
  "anchorMenu": "CustomMenu-module__CNRCOvKg__v1-2-16",
  "anchorMenuItem": "CustomMenu-module__oICdg9VX__v1-2-16",
  "defaultMenuItem": "CustomMenu-module__TNQGZNnJ__v1-2-16"
};
var CustomMenu = function (_a) {
  var _b;
  var items = _a.items,
    onClick = _a.onClick,
    dragProps = _a.dragProps,
    variant = _a.variant,
    metadata = _a.metadata;
  var initialList = useMemo(function () {
    var isActive = items.some(function (_a) {
      var active = _a.active;
      return active;
    });
    if (!items.length) {
      return items;
    }
    if (isActive) {
      return items;
    }
    var firstItem = items[0],
      restItems = items.slice(1);
    return __spreadArray([__assign(__assign({}, firstItem), {
      active: true
    })], restItems, true);
  }, [items]);
  var _c = useState(initialList),
    list = _c[0],
    setList = _c[1];
  var isAnchor = variant === MenuVariants.ANCHOR;
  var onClickHandler = useCallback(function (_a) {
    _a.active;
    var rest = __rest(_a, ["active"]);
    setList(function (prevList) {
      return prevList.map(function (listItem) {
        return __assign(__assign({}, listItem), {
          active: listItem.key === rest.key
        });
      });
    });
    onClick(rest);
  }, [onClick]);
  useEffect(function () {
    setList(initialList);
  }, [initialList]);
  return /*#__PURE__*/React.createElement("div", __assign({
    className: classnames(styles.droppableZoneWrapper, (_b = {}, _b[styles.anchorMenu] = isAnchor, _b))
  }, parseMetadata(metadata)), /*#__PURE__*/React.createElement(DragAndDropList, {
    list: list,
    dragProps: dragProps,
    listItemClassName: isAnchor ? styles.anchorMenuItem : styles.defaultMenuItem,
    renderItem: function (_a) {
      var item = _a.item,
        dragRef = _a.dragRef,
        isOver = _a.isOver,
        canDrop = _a.canDrop;
      return /*#__PURE__*/React.createElement(MenuItem, {
        item: item,
        isSelected: !!item.active,
        onClick: onClickHandler,
        dragRef: dragRef,
        isOver: isOver,
        canDrop: canDrop,
        variant: variant
      });
    }
  }));
};
export { CustomMenu };
//# sourceMappingURL=index.js.map
